"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _generator = _interopRequireDefault(require("@babel/generator"));

var _parser = require("@babel/parser");

var _helperModuleImports = require("@babel/helper-module-imports");

var _helpers = require("./helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let cssHint;

function _default({
  types: t
}) {
  return {
    name: 'styled-props-conditions',
    visitor: {
      Program() {
        cssHint = null;
      },

      TaggedTemplateExpression(path) {
        const {
          quasis
        } = path.node.quasi;
        const quasisRaws = quasis.map(({
          value
        }) => value.raw);
        const conditionalBlocks = (0, _helpers.findConditionalBlocks)(quasisRaws);
        const replaceData = [];
        if (!conditionalBlocks.length) return;

        if (!cssHint) {
          cssHint = (0, _helperModuleImports.addNamed)(path, 'css', 'styled-components');
        }

        conditionalBlocks.forEach(conditionalBlock => {
          const {
            blockStart,
            blockEnd,
            bodyStart,
            chunkStart,
            chunkEnd,
            condition
          } = conditionalBlock;
          replaceData.push({
            quasiIndex: chunkEnd,
            start: blockEnd,
            del: 0,
            str: '`'
          }, {
            quasiIndex: chunkStart,
            start: blockStart,
            del: bodyStart - blockStart + 1,
            str: (0, _helpers.createExpression)(condition, cssHint.name)
          });
        });
        const sortedArray = replaceData.sort((a, b) => b.quasiIndex - a.quasiIndex || b.start - a.start);
        sortedArray.forEach(({
          quasiIndex,
          start,
          del,
          str
        }) => {
          const arr = [...quasisRaws[quasiIndex]];
          arr.splice(start, del, str);
          quasisRaws[quasiIndex] = arr.join('');
        });
        quasisRaws.forEach((raw, index) => {
          path.node.quasi.quasis[index] = t.templateElement({
            raw
          });
        });
        const {
          code
        } = (0, _generator.default)(path.node);
        const {
          expression
        } = (0, _parser.parse)(code).program.body[0];
        path.replaceWith(expression);
      }

    }
  };
}